'use strict';

define(function (require) {
    var ko = require('knockout');
    require('knockout.validation');
    var environment = require('environment');
    var TaskViewModel = require('taskViewModel');
    var actionLogHelper = require('actionLogHelper');
    var locHelper = require('locHelper');
    var stringHelper = require('stringHelper');
    var constants = require('constants');
    var incidentHelper = require('incidentHelper');
    var moment = require('moment');

    /**
     * Resolves an Incident
     * @constructor
     * @param {Object} context - The context for the form
     */
    function ViewModel(context) {
        var self = this;
        TaskViewModel.call(self, context);
        self.maxLength = 4000;
        self.resolutions = context.enumStore().IncidentResolutionCategoryEnum;
        self.statuses = context.enumStore().IncidentStatusEnum;
        self.status = ko.observable().extend({
            required: {
                params: true
            }
        });
        self.resolution = ko.observable().extend({
            required: {
                onlyIf: function () {
                    return self.status() === 'IncidentStatusEnum.Resolved';
                }
            }
        });
        self.resolutionDescription = ko.observable('').extend({
            maxLength: self.maxLength,
            required: {
                onlyIf: function () {
                    return self.status() === 'IncidentStatusEnum.Resolved';
                }
            }
        });

        // Do initial validation
        self.isValid = function () {
            var validLoading = $.Deferred();

            // Reject deffered with status level and message if this incident is not suitible for manipulation by this task.
            // return validLoading.reject({
            //     status: 'info',
            //     message: 'No no no no....'
            // });

            return validLoading.resolve().promise();
        };

        /**
         * Resolves the incident
         * @returns {Object} - Promise 
         */
        self.execute = function () {
            if (self.status() === 'IncidentStatusEnum.Resolved') {
                var resolution = this.resolution();
                var resolutionDescription = this.resolutionDescription();
                context.formData().ResolutionCategory(resolution);
                context.formData().ResolutionDescription(resolutionDescription);
                context.formData().ResolvedDate(moment().toISOString());
                context.formData().ResolvedByUser(environment.loggedOnUser);

                //Spawn an action log item
                var entry = actionLogHelper.getActionEntry('ActionLogs',
                    constants.enums.System_WorkItem_ActionLogEnum_RecordResolved,
                    resolutionDescription,
                    context.typeMetadata());

                //Add comment to action log
                context.formData().ActionLogs.push(entry);
            }
            context.formData().Status(self.status());
            return $.Deferred().resolve();
        };
    }

    return ViewModel;
});