'use strict';

define(function (require) {
    var TaskViewModel = require('taskViewModel');
    var Router = require('router');
    var ko = require('knockout');
    var entityHelper = require('entityHelper');
    var metadataCache = require('metadataCache');
    var EntityFactory = require('entityFactory');
    var environment = require('environment');
    require('knockout.validation');

    const System_WorkItem_ServiceRequestProjection = 'e44b7c06-590d-64d6-56d2-2219c5e763e0';
    const System_WorkItem_ServiceRequest = '04b69835-6343-4de2-4b19-6be08c612989';
    const ServiceRequestStatusEnum_New = 'ServiceRequestStatusEnum.New';

    function ViewModel(context, params) {
        var self = this;
        TaskViewModel.call(self, context);
        // Spawn a new projection instance or fetch an existing
        self.isNew = true;
        // Which TypeProjection should be used to wrap the new or existing instance
        self.typeProjectionId = System_WorkItem_ServiceRequestProjection;
        // Set target class for new instance, only used with isNew = true
        self.classId = System_WorkItem_ServiceRequest;
        // Allows you to spawn a new entityt with a template, only used with isNew = true
        self.templateId = '';
        // Allows you to load sub projections into the main projection
        self.subTypeProjectionsToLoad = [];

        self.typeMetadata = null;
        self.srId = '';

        // Since this task doesn't have a UI the execute method is executed immediately.
        self.execute = function () {
            // Only used when isNew = false
            var entityId = '';

            return metadataCache.getFormProjectionMetadata(self.typeProjectionId, self.classId, self.subTypeProjectionsToLoad, true).then(function (typeMetadata) {
                self.typeMetadata = typeMetadata;
                return entityHelper.getProjectionInstance(self.isNew,
                    self.typeProjectionId,
                    self.classId,
                    self.templateId,
                    self.subTypeProjectionsToLoad,
                    entityId);
            }).then(function (projectionInstance) {
                var serviceRequest = EntityFactory.createTrackedFormEntity(projectionInstance,
                    projectionInstance.$ClassId$,
                    '',
                    self.typeMetadata.relationshipTypes);

                //Replace identifers of possible analyst comments and/or action log items from template.
                if (self.isNew && self.templateId) {
                    entityHelper.replaceTemplateResourceIdentifiers(serviceRequest);
                }

                // Get incident data through the context object.
                var incident = context.formData();
                incident.Status('IncidentStatusEnum.Closed');

                // Get settings through environment.settings.
                // var serviceRequestSettings = environment.settings.workItemSettings.serviceRequestSettings;

                // Copy data from incident to Service Request
                var title = "#Copied: " + incident.Title();
                serviceRequest.Title(title.substring(0, 256));
                var description = "#Copied: " + incident.Description();
                serviceRequest.Description(description.substring(0, 4000));
                serviceRequest.AboutConfigItem(incident.AffectedConfigItems());
                serviceRequest.Status(ServiceRequestStatusEnum_New);
                serviceRequest.AffectedUser(incident.AffectedUser());
                serviceRequest.ContactMethod(incident.ContactMethod());

                // Store the identifier of the service request so that we can navigate to the SR after creating it.
                self.srId = serviceRequest.$Id$();

                // Convert the service request to suitible format before submitting
                var formData = entityHelper.convertJsonEntityToApiEntity(serviceRequest);
                return entityHelper.createOrUpdateProjection(formData);
            }).then(function () {
                // Save the incident
                return context.submit();
            }).then(function () {
                // We need to reset dirty flag after submit before trying to close form. If not we'll prompt the user asking about unsaved changes that has actually been saved.
                context.resetDirtyFlag();
                // Close the incident form
                return context.close();
            }).then(function () {
                // Open the new service request form
                Router.open('/entity/{0}'.replace('{0}', self.srId));
            });
        };
    }

    return ViewModel;
});