'use strict';

define(function (require) {
    var TaskViewModel = require('taskViewModel');
    var ko = require('knockout');
    var metadataCache = require('metadataCache');
    require('knockout.validation');
    require('jquery');
    const windowsComputerClass = 'ea99500d-8d52-fc52-b5a5-10dcd1e9d2bd';

    function RdpTaskViewModel(context, params) {
        var self = this;
        TaskViewModel.call(self, context);

        // Generate a RDP file and prompt for download
        self.generateFileForComputer = function (computerCi) {
            var filename = computerCi.$DisplayName$() + '.rdp';
            var text = "full address:s:" + computerCi.$DisplayName$() + "\n" +
                "prompt for credentials:i:1\n" +
                "administrative session:i:1";
            var element = document.createElement('a');
            element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(text));
            element.setAttribute('download', filename);
            element.style.display = 'none';
            document.body.appendChild(element);
            element.click();
            document.body.removeChild(element);
        };

        self.processConfigItem = function (configItem) {
            return metadataCache.getClassMetadata(configItem.$ClassId$()).then(function (classMetaData) {
                // If this config item is a type of Windows Computer, create an RDP file.
                var mpClass = classMetaData.type;
                while (mpClass) {
                    if (mpClass.id === windowsComputerClass) {
                        self.generateFileForComputer(configItem);
                        break;
                    }
                    mpClass = mpClass.base;
                }
            });
        };

        // Since this task doesn't have a UI the execute method is executed immediately.
        self.execute = function () {
            // Enable below for Chrome to attach debugger.
            //debugger;

            var deffered = $.Deferred();

            // Get affected configuration items.
            var $affectedConfigItems = context.formData().AffectedConfigItems();

            var jobs = [];
            // Generate an RDP file for all Configuration Items that is based on a class that derives from the Windows Computer Class.
            $affectedConfigItems.forEach(ci => {
                jobs.push(self.processConfigItem(ci));
            });

            // When all CIs are processed, resolve the deffered to tell WebFront that the task is done.
            $.when(jobs).then(deffered.resolve());

            // Return a Deffered's Promise object that is resolved once the task has completed. For more information: http://api.jquery.com/Types/#Deferred
            return deffered.promise();
        };
    }

    return RdpTaskViewModel;
});