'use strict';

define(function (require) {
    var TaskViewModel = require('taskViewModel');
    var ko = require('knockout');
    var metadataCache = require('metadataCache');
    require('knockout.validation');
    require('jquery');

    /**
     * The view model for the task.
     * @param {Object} context Represents the form/view etc. the task is executed for (e.g. the Incident Form).
     * @param {Object} args If you define arguments in the task declaration (_tasks.json) those are exposed on the args object.
     */
    function ViewModel(context, args) {
        var self = this;
        // This should always be included in your custom task to setup the view model according to internal requirements.
        TaskViewModel.call(self, context);

        // To automatically attach the debugger in your browsers developer tools (F12), enable the line below.
        //debugger;

        /**
         * Executed once the user presses OK button when the task has a UI template, in the
         * case of no UI template the function is executed emediatly after the view model is initiated.
         * @returns {Promise} Deffered's promise object: http://api.jquery.com/Types/#Deferred
        */
        self.execute = function () {
            // Prepare a Deffered object
            var deffered = $.Deferred();
            // Do changes to the work-/config item.
            context.formData().Description('You ran the sample task :)');
            // Resolve deffered once you've completed the task. If you're running an async process this will make sure the UI is locked until the deffered is resolved.
            deffered.resolve();
            // Return a Deffered's Promise object that is resolved once the task has completed.
            return deffered.promise();
        };
    }

    return ViewModel;
});